#!/usr/bin/python
#
#Utility to assist in measurement of effective noise bandwidth (ENB)
#of SSB receiver.
#Reads saved spectrum log from Spectrogram V11 of receiver response to
#white noise input, and calculates ENB.
#
# Copyright: Owen Duffy 06-04-2005. All rights reserved.

import os,sys
import getopt
import re
from array import *

print('\nsl2enb.py\n')
f3=0
locut=0

def usage():
  print("\nUsage: "+sys.argv[0]+" [-c ReferenceFrequency][-l LoCutFreq] <SpectrogramLogFile>\n\n");
  sys.exit(1)

try:
  opts,args=getopt.getopt(sys.argv[1:],"c:hl:v",["help","RefFreq=","LoCutFreq="])
except getopt.GetoptError as err:
  # print help information and exit:
  print(err)  # will print something like "option -a not recognized"
  usage()
verbose=False
for o,a in opts:
  if o=="-v":
    verbose = True
  elif o in ("-c", "--RefFreq"):
    f3=float(a)
  elif o in ("-h", "--help"):
    usage()
    sys.exit()
  elif o in ("-l", "--LoCutFreq"):
    locut=float(a)
  else:
    assert False, "unhandled option"
try:
  filename=args[0]
except:
  usage()

f2=1000
obs=[]
pat=re.compile("^([\d\.]*) Hz Frequency Resolution$")
f=open(filename)
i=0
while i<5:
  i+=1
  line=f.readline()
  if pat.match(line):
    res=float(re.sub('^([\d\.]*) Hz Frequency Resolution$','\\1',line, count=0, flags=0))
    break

for i in range(6): #skip header
    f.readline()

midtot=0
midn=0
while 1 :
  line=f.readline()
  if line=='': break
  num,hz,db=line.split()
  num=int(num)
  hz=float(hz)
  db=float(db)
  if hz<locut: continue #low cut filter
  pwr=10**(db/10)
  if hz>(f2-20) and hz<(f2+20):
    midtot+=pwr
    midn+=1
  obs.append([pwr,hz])
f.close

#throw away the last obs, it is sometimes defective
obs.pop()

pwrtot=0;
for ob in obs:
  pwrtot+=ob[0]

#1KHz (f2)
f2pwr=midtot/midn
enb2=pwrtot/f2pwr*res;

#find passband centre (f1)
obs.sort(key=lambda x:x[0],reverse=True) #sort by pwr
pwrmax=obs[0][0]

fl=100000
fu=0
for ob in obs:
  if ob[0]<pwrmax/10: break
  if fl>ob[1]: fl=ob[1];
  if fu<ob[1]: fu=ob[1];

#f1 is midway between fl and fu
f1=(fl+fu)/2

fl=100000
fu=0
for ob in obs:
  if ob[0]<pwrmax/4: break
  if fl>ob[1]: fl=ob[1];
  if fu<ob[1]: fu=ob[1];

#show ENB wrt centre freq
print("\nFilter -6dB response: {0:0.0f}-{1:0.0f}Hz={2:0.0f}Hz.\n".format(fl,fu,fu-fl))

#rescan the data to get the gain at f1.
midtot=0;
midn=0;
for ob in obs:
  if ob[1]>f1-20 and ob[1]<f1+20:
    midtot+=ob[0]
    midn+=1

#passband centre (f1)
f1pwr=midtot/midn
enb1=pwrtot/f1pwr*res

f3pwr=0
if f3>0:
  #rescan the data to get the gain at f3.
  midtot=0
  midn=0
  for ob in obs:
    if ob[1]>f3-20 and ob[1]<f3+20:
      midtot+=ob[0]
      midn+=1
  
  #specified freq (f3)
  f3pwr=midtot/midn
  enb3=pwrtot/f3pwr*res

#show ENB wrt centre freq
print("\nENB={0:0.0f}Hz with respect to gain at {1:0.0f}Hz (passband centre frequency).\n\n".format(enb1,f1));
#if gain at f1 (1KHz) is greater than at centre freq -10dB, show ENB wrt f1
if f2pwr>f1pwr/10:
  print("ENB={0:0.0f}Hz with respect to gain at {1:0.0f}Hz.\n\n".format(enb2,f2))
#if gain at f3 (command line arg) is greater than at centre freq -10dB, show ENB wrt f3
if f3pwr>f1pwr/10:
  print("ENB={0:0.0f}Hz with respect to gain at {1:0.0f}Hz (specified).\n\n".format(enb3,f3))




